<?php

/**
 * Copyright distrib (2018)
 *
 * contact@opendistrib.net
 *
 * Ce logiciel est un programme informatique servant à aider les producteurs
 * à distribuer leur production en circuits courts.
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */

use yii\helpers\Html;
use yii\bootstrap\ActiveForm;
use common\models\Product;
use yii\helpers\ArrayHelper;
use common\models\TaxRate;
use common\models\Producer;
use common\helpers\GlobalParam;

/* @var $this yii\web\View */
/* @var $model app\models\Produit */
/* @var $form yii\widgets\ActiveForm */
?>

<div class="product-form">

        <?php $form = ActiveForm::begin([
                'enableClientValidation' => false,
                'options' => ['enctype' => 'multipart/form-data']
        ]); ?>

    <div>
        <div class="col-md-8">
                <?= $form->field($model, 'active')->radioList([1 => 'Oui', 0 => 'Non']) ?>
                <?= $form->field($model, 'name')->textInput(['maxlength' => 255]) ?>
                <?= $form->field($model, 'description')->textInput(['maxlength' => 255]) ?>
                <?= $form->field($model, 'recipe')->textarea()->label('Description longue') ?>

                <?= $form->field($model, 'unit')
                        ->dropDownList(ArrayHelper::map(Product::$unitsArray, 'unit', 'wording'))
                        ->label('Unité (pièce, poids ou volume)'); ?>


                <?php

                //Récupère la tva par défaut du producteur courant
                $producer = \common\helpers\GlobalParam::getCurrentProducer();
                $taxRateDefault = $producer->taxRate;

                $taxRateNamesArray = array_merge(array(0 => 'Tva par défaut'), ArrayHelper::map(TaxRate::find()->all(), 'id', function ($model) {
                        return $model->name;
                }));
                $taxRateValuesArray = array_merge(array(0 => $taxRateDefault->value), ArrayHelper::map(TaxRate::find()->all(), 'id', function ($model) {
                        return $model->value;
                }));
                foreach ($taxRateValuesArray as $key => $taxRateValue) {
                        $taxRateValuesArrayFormatted[$key] = array('data-tax-rate-value' => $taxRateValue);
                }

                ?>

                <?php if($taxRateDefault->value != 0): ?>
                        <?= $form->field($model, 'id_tax_rate')->dropDownList($taxRateNamesArray, ['options' => $taxRateValuesArrayFormatted])->label('Taxe'); ?>
                <?php endif; ?>

                <?= $form->field($model, 'price', [
                        'template' => '
                        <div class="row">
                        <div class="col-xs-6">
                            <label for="product-price" class="control-label without-tax"></label>
                            <div class="input-group">
                            {input}<span class="input-group-addon"><span class="glyphicon glyphicon-euro"></span></span>
                            </div>
                        </div>
                        <div class="col-xs-6">
                            <label for="product-price-with-tax" class="control-label with-tax"></label>
                            <div class="input-group">
                                <input type="text" id="product-price-with-tax" class="form-control" name="" value="">
                                <span class="input-group-addon"><span class="glyphicon glyphicon-euro"></span></span>
                            </div>
                        </div>
                        </div>
                        
                    ',
                ]) ?>
                <?= $form->field($model, 'step')->textInput()->hint('Définit ce qui est ajouté ou enlevé lors des changements de quantité.') ?>

                <?= $form->field($model, 'weight')->textInput()->label('Poids (g)') ?>

                <div class="col-md-3">
                        <?= $form->field($model, 'quantity_max')->textInput() ?>
                </div>
                <div class="col-md-3">
                        <?= $form->field($model, 'quantity_max_monday')->textInput() ?>
                </div>
                <div class="col-md-3">
                        <?= $form->field($model, 'quantity_max_tuesday')->textInput() ?>
                </div>
                <div class="col-md-3">
                        <?= $form->field($model, 'quantity_max_wednesday')->textInput() ?>
                </div>
                <div class="col-md-3">
                        <?= $form->field($model, 'quantity_max_thursday')->textInput() ?>
                </div>
                <div class="col-md-3">
                        <?= $form->field($model, 'quantity_max_friday')->textInput() ?>
                </div>
                <div class="col-md-3">
                        <?= $form->field($model, 'quantity_max_saturday')->textInput() ?>
                </div>
                <div class="col-md-3">
                        <?= $form->field($model, 'quantity_max_sunday')->textInput() ?>
                </div>
                <div class="clr"></div>

                <?php
                if (!$model->isNewRecord) {
                        echo $form->field($model, 'apply_distributions')
                                ->checkbox()
                                ->hint('Sélectionnez cette option si vous souhaitez que ces modifications (actif / non actif, quantité max) soient répercutées dans les distributions à venir déjà initialisées.');
                }
                ?>
        </div>
        <div class="col-md-4">
                <?= $form->field($model, 'photo')->fileInput() ?>
                <?php
                if (strlen($model->photo)) {
                        $url = Yii::$app->urlManagerProducer->getHostInfo() . '/' . Yii::$app->urlManagerProducer->baseUrl . '/uploads/' . $model->photo;
                        $url = str_replace('//uploads','/uploads', $url) ;
                        echo '<img class="photo-product" src="' . $url . '"  width="200px" /><br />';
                        echo '<input type="checkbox" name="delete_photo" id="delete_photo" /> <label for="delete_photo">Supprimer la photo</label><br /><br />';
                }
                ?>

            <div id="days-production">
                <h2>Jours de distribution</h2>
                    <?= $form->field($model, 'monday')->checkbox() ?>
                    <?= $form->field($model, 'tuesday')->checkbox() ?>
                    <?= $form->field($model, 'wednesday')->checkbox() ?>
                    <?= $form->field($model, 'thursday')->checkbox() ?>
                    <?= $form->field($model, 'friday')->checkbox() ?>
                    <?= $form->field($model, 'saturday')->checkbox() ?>
                    <?= $form->field($model, 'sunday')->checkbox() ?>
            </div>
            <div class="clr"></div>
        </div>
        <div class="clr"></div>
    </div>

        <?= $form->field($model, 'id_producer')->hiddenInput()->label('') ?>

    <div class="form-group">
            <?= Html::submitButton($model->isNewRecord ? 'Ajouter' : 'Modifier', ['class' => $model->isNewRecord ? 'btn btn-success' : 'btn btn-primary']) ?>
    </div>

        <?php ActiveForm::end(); ?>

</div>
